;
;
;
;    Resea (x86/assembly.asm)
;    Mafice <mafice@mflabo.net>
;
;
;

[bits 32]



;
;  Function: asm_startupInterpreter
;
;    インタプリタを起動する
;
;  Parameters:
;
;    第1引数  - コードのアドレス
;    第2引数  - コードセグメントレジスタ
;    第3引数  - データセグメントレジスタ
;    第4引数  - スタックポインタ
;    第5引数   - RFLAGSレジスタ
;
;  Returns:
;
;    なし
;

global asm_startupInterpreter
asm_startupInterpreter:


mov edi, [esp+4]
mov esi, [esp+8]
mov edx, [esp+12]
mov ecx, [esp+16]
mov eax, [esp+20]

add edx, 3
add esi, 3

push dword edx   ;SS
push dword ecx   ;ESP
push dword eax   ;EFLAGS
push dword esi   ;CS
push dword edi   ;EIP

mov  ax, dx
mov  ds, ax
mov  es, ax
mov  fs, ax
mov  gs, ax

iretd




;
;  Function: asm_hlt
;
;    HLT命令を実行する
;
;  Parameters:
;
;    なし
;
;  Returns:
;
;    なし
;

global asm_hlt
asm_hlt:

hlt
ret




;
;  Function: asm_cli
;
;    CLI命令を実行する
;
;  Parameters:
;
;    なし
;
;  Returns:
;
;    なし
;

global asm_cli
asm_cli:

cli
ret




;
;  Function: asm_sti
;
;    STI命令を実行する
;
;  Parameters:
;
;    なし
;
;  Returns:
;
;    なし
;

global asm_sti
asm_sti:

sti
ret




;
;  Function: asm_out8
;
;    OUT命令を実行する
;
;  Parameters:
;
;    第1引数 - データ
;    第2引数  - ポート
;
;  Returns:
;
;    なし
;

global asm_out8
asm_out8:

mov  dx, [esp+4]
mov  al, [esp+8]
out  dx, al
ret




;
;  Function: asm_in8
;
;    IN命令を実行する
;
;  Parameters:
;
;    第1引数  - ポート
;
;  Returns:
;
;    読み取ったデータ
;

global asm_in8
asm_in8:

mov  eax, 0
mov  dx,  [esp+4]
in   al,  dx
ret




;
;  Function: asm_lgdt
;
;    LGDT命令を実行する
;
;  Parameters:
;
;    第1引数  - LGTRの内容へのアドレス
;
;  Returns:
;
;    なし
;

global asm_lgdt
asm_lgdt:

mov  eax, [esp+4]
lgdt [eax]
ret




;
;  Function: asm_lidt
;
;    LIDT命令を実行する
;
;  Parameters:
;
;    第1引数  - LDTRの内容へのアドレス
;
;  Returns:
;
;    なし
;

global asm_lidt
asm_lidt:

mov  eax, [esp+4]
lidt [eax]
ret




;
;  Function: asm_ltr
;
;    LTR命令を実行する
;
;  Parameters:
;
;    第1引数  - TSSセレクタ
;
;  Returns:
;
;    なし
;

global asm_ltr
asm_ltr:

ltr [esp+4]
ret




;
;  Function: asm_setCR3
;
;    CR3レジスタをセットする
;
;  Parameters:
;
;    第1引数  - CR3レジスタのデータ
;
;  Returns:
;
;    なし
;

global asm_setCR3
asm_setCR3:

mov  eax, [esp+4]
mov  cr3, eax
ret




;
;  Function: asm_loadCR2
;
;    CR2レジスタを読み込む
;
;  Parameters:
;
;    なし
;
;  Returns:
;
;    CR2レジスタのデータ
;

global asm_loadCR2
asm_loadCR2:

mov eax, cr2
ret

