/*
 *
 *
 *    Resea (x86_64/api.c)
 *    Mafice <mafice@mflabo.net>
 *
 *
 */

#include "kernel.h"



/*
*  Function: APIEntry
*
*    APIエントリ
*
*  Parameters:
*
*    rax -  RAXレジスタ
*    rbx -  RBXレジスタ
*    rcx -  RCXレジスタ
*    rdx -  RDXレジスタ
*    rdi -  RDIレジスタ
*    rsi -  RSIレジスタ
*
*  Returns:
*
*    各APIに依存します。
*
*/

u64 APIEntry (u64 rax, u64 rbx, u64 rcx, u64 rdx, u64 rdi, u64 rsi){



  //putchar
  if(rax == API_putchar){

    printchar((char) rdi, Screen.textColor|Screen.bgColor);
    return 0;
  }


  //out
  if(rax == API_out){

    if(rcx == 8)
      asm_out8((u16) rdi, (u8) rbx);

    return 0;
  }


  //in
  if(rax == API_in){

    if(rcx == 8)
      return asm_in8((u16) rsi);
  }


  //setISR
  if(rax == API_setISR){

    if((0 <= rsi) && (rsi <= ISR_MAX)){


      Interrupt.isr[rsi] = rdx;
      return true;

    }else{

      return false;
    }
  }


  //waitInterrupt
  if(rax == API_waitInterrupt){

    asm_sti();
    asm_hlt();
    return 0;
  }


  return 0;
}

