/*
 *
 *
 *    Resea (x86_64/boot4.c)
 *    Mafice <mafice@mflabo.net>
 *
 *
 */

#include "kernel.h"



/*
*  Function: boot4
*
*    エントリーポイント
*
*  Parameters:
*
*    なし
*
*  Returns:
*
*    なし
*
*/

void boot4 (void){

 struct st_GDTR  *GDTR  = (struct st_GDTR *)  GDTR_ADDRESS;
 struct st_IDTR  *IDTR  = (struct st_IDTR *)  IDTR_ADDRESS;
 struct st_TSS   *TSS   = (struct st_TSS  *)  TSS_ADDRESS;
 count_t  i;



  //==========================================================================
  //  Screen構造体の初期化
  //==========================================================================


    Screen.VRAMAddress = VRAM_ADDRESS;
    Screen.x           = 0;
    Screen.y           = 0;
    Screen.textColor   = CUI_DEFAULT_TEXT_COLOR;
    Screen.bgColor     = CUI_DEFAULT_BG_COLOR;
    Screen.xMax        = SCREEN_X_MAX;
    Screen.yMax        = SCREEN_Y_MAX;



  //==========================================================================
  //  ウェルカムメッセージ
  //==========================================================================


    Screen.x = Screen.xMax-((Screen.xMax/2)+strlen("starting ...")/2)-1;
    Screen.y = Screen.yMax-2;
    print("starting ...");



  //==========================================================================
  //  GDTの設定
  //==========================================================================


    setSegDescriptor(GDT_KERNEL_CODE_SELECTOR, 0x00000000,  0xffffffff, GDT_KERNEL_CODESEG);
    setSegDescriptor(GDT_KERNEL_DATA_SELECTOR, 0x00000000,  0xffffffff, GDT_KERNEL_DATASEG);
    setSegDescriptor(GDT_USER_CODE_SELECTOR,   0x00000000,  0xffffffff, GDT_USER_CODESEG);
    setSegDescriptor(GDT_USER_DATA_SELECTOR,   0x00000000,  0xffffffff, GDT_USER_DATASEG);
    setTSSDescriptor(GDT_TSS_SELECTOR,         TSS_ADDRESS, GDT_TSS);


    GDTR->length   = GDT_LENGTH;
    GDTR->address  = GDT_ADDRESS;

    asm_lgdt(GDTR_ADDRESS);



  //==========================================================================
  //  IDTの設定
  //==========================================================================


    // 例外ハンドラ
    setIntDescriptor(0,  GDT_KERNEL_CODE_SELECTOR, (address_t) &asm_exception0,  IDT_INT_HANDLER);
    setIntDescriptor(1,  GDT_KERNEL_CODE_SELECTOR, (address_t) &asm_exception1,  IDT_INT_HANDLER);
    setIntDescriptor(2,  GDT_KERNEL_CODE_SELECTOR, (address_t) &asm_exception2,  IDT_INT_HANDLER);
    setIntDescriptor(3,  GDT_KERNEL_CODE_SELECTOR, (address_t) &asm_exception3,  IDT_INT_HANDLER);
    setIntDescriptor(4,  GDT_KERNEL_CODE_SELECTOR, (address_t) &asm_exception4,  IDT_INT_HANDLER);
    setIntDescriptor(5,  GDT_KERNEL_CODE_SELECTOR, (address_t) &asm_exception5,  IDT_INT_HANDLER);
    setIntDescriptor(6,  GDT_KERNEL_CODE_SELECTOR, (address_t) &asm_exception6,  IDT_INT_HANDLER);
    setIntDescriptor(7,  GDT_KERNEL_CODE_SELECTOR, (address_t) &asm_exception7,  IDT_INT_HANDLER);
    setIntDescriptor(8,  GDT_KERNEL_CODE_SELECTOR, (address_t) &asm_exception8,  IDT_INT_HANDLER);
    setIntDescriptor(9,  GDT_KERNEL_CODE_SELECTOR, (address_t) &asm_exception9,  IDT_INT_HANDLER);
    setIntDescriptor(10, GDT_KERNEL_CODE_SELECTOR, (address_t) &asm_exception10, IDT_INT_HANDLER);
    setIntDescriptor(11, GDT_KERNEL_CODE_SELECTOR, (address_t) &asm_exception11, IDT_INT_HANDLER);
    setIntDescriptor(12, GDT_KERNEL_CODE_SELECTOR, (address_t) &asm_exception12, IDT_INT_HANDLER);
    setIntDescriptor(13, GDT_KERNEL_CODE_SELECTOR, (address_t) &asm_exception13, IDT_INT_HANDLER);
    setIntDescriptor(14, GDT_KERNEL_CODE_SELECTOR, (address_t) &asm_exception14, IDT_INT_HANDLER);
    setIntDescriptor(15, GDT_KERNEL_CODE_SELECTOR, (address_t) &asm_exception15, IDT_INT_HANDLER);
    setIntDescriptor(16, GDT_KERNEL_CODE_SELECTOR, (address_t) &asm_exception16, IDT_INT_HANDLER);
    setIntDescriptor(17, GDT_KERNEL_CODE_SELECTOR, (address_t) &asm_exception17, IDT_INT_HANDLER);
    setIntDescriptor(18, GDT_KERNEL_CODE_SELECTOR, (address_t) &asm_exception18, IDT_INT_HANDLER);
    setIntDescriptor(19, GDT_KERNEL_CODE_SELECTOR, (address_t) &asm_exception19, IDT_INT_HANDLER);


    //ハードウェア割り込みハンドラ
    setIntDescriptor(IDT_HW_INT_SELECTOR+0,  GDT_KERNEL_CODE_SELECTOR, (address_t) &asm_hwIntHandler0,   IDT_INT_HANDLER);
    setIntDescriptor(IDT_HW_INT_SELECTOR+1,  GDT_KERNEL_CODE_SELECTOR, (address_t) &asm_hwIntHandler1,   IDT_INT_HANDLER);
    setIntDescriptor(IDT_HW_INT_SELECTOR+2,  GDT_KERNEL_CODE_SELECTOR, (address_t) &asm_hwIntHandler2,   IDT_INT_HANDLER);
    setIntDescriptor(IDT_HW_INT_SELECTOR+3,  GDT_KERNEL_CODE_SELECTOR, (address_t) &asm_hwIntHandler3,   IDT_INT_HANDLER);
    setIntDescriptor(IDT_HW_INT_SELECTOR+4,  GDT_KERNEL_CODE_SELECTOR, (address_t) &asm_hwIntHandler4,   IDT_INT_HANDLER);
    setIntDescriptor(IDT_HW_INT_SELECTOR+5,  GDT_KERNEL_CODE_SELECTOR, (address_t) &asm_hwIntHandler5,   IDT_INT_HANDLER);
    setIntDescriptor(IDT_HW_INT_SELECTOR+6,  GDT_KERNEL_CODE_SELECTOR, (address_t) &asm_hwIntHandler6,   IDT_INT_HANDLER);
    setIntDescriptor(IDT_HW_INT_SELECTOR+7,  GDT_KERNEL_CODE_SELECTOR, (address_t) &asm_hwIntHandler7,   IDT_INT_HANDLER);
    setIntDescriptor(IDT_HW_INT_SELECTOR+8,  GDT_KERNEL_CODE_SELECTOR, (address_t) &asm_hwIntHandler8,   IDT_INT_HANDLER);
    setIntDescriptor(IDT_HW_INT_SELECTOR+9,  GDT_KERNEL_CODE_SELECTOR, (address_t) &asm_hwIntHandler9,   IDT_INT_HANDLER);
    setIntDescriptor(IDT_HW_INT_SELECTOR+10, GDT_KERNEL_CODE_SELECTOR, (address_t) &asm_hwIntHandler10,  IDT_INT_HANDLER);
    setIntDescriptor(IDT_HW_INT_SELECTOR+11, GDT_KERNEL_CODE_SELECTOR, (address_t) &asm_hwIntHandler11,  IDT_INT_HANDLER);
    setIntDescriptor(IDT_HW_INT_SELECTOR+12, GDT_KERNEL_CODE_SELECTOR, (address_t) &asm_hwIntHandler12,  IDT_INT_HANDLER);
    setIntDescriptor(IDT_HW_INT_SELECTOR+13, GDT_KERNEL_CODE_SELECTOR, (address_t) &asm_hwIntHandler13,  IDT_INT_HANDLER);
    setIntDescriptor(IDT_HW_INT_SELECTOR+14, GDT_KERNEL_CODE_SELECTOR, (address_t) &asm_hwIntHandler14,  IDT_INT_HANDLER);
    setIntDescriptor(IDT_HW_INT_SELECTOR+15, GDT_KERNEL_CODE_SELECTOR, (address_t) &asm_hwIntHandler15,  IDT_INT_HANDLER);


    //API
    setIntDescriptor(IDT_API_SELECTOR, GDT_KERNEL_CODE_SELECTOR, (address_t) &asm_APIEntry, IDT_API);



    //割り込みハンドラのアドレスをクリア
    for(i=0; i < ISR_MAX; i++)
      Interrupt.isr[i] = 0;

    IDTR->length   = IDT_LENGTH;
    IDTR->address  = IDT_ADDRESS;

    asm_lidt(IDTR_ADDRESS);



  //==========================================================================
  //  TSS初期化
  //==========================================================================


    TSS->rsp0      = TSS_RSP0;
    TSS->ist[1]    = TSS_IST1;
    TSS->ioMapBase = ((u64) &TSS->ioPermMap) - ((u64) TSS);

    for(i=0; i < 8192; i++)
      TSS->ioPermMap[i] = 0;

    asm_ltr(GDT_TSS_SELECTOR);



  //==========================================================================
  //  割り込みの受付を開始
  //==========================================================================


    asm_out8(IOPORT_INTMASK_MASTER, INTERRUPT_MASK_MASTER);
    asm_out8(IOPORT_INTMASK_SLAVE,  INTERRUPT_MASK_SLAVE);



  //==========================================================================
  //  インタプリタをINTEPRETER_ADDRESSへコピー
  //==========================================================================


   u8 *p= (u8 *) INTERPRETER_ADDRESS;
   u8 *q= (u8 *) INTERPRETER_IMG_ADDRESS;


    for(i=0; i < INTERPRETER_SIZE; i++){

      p[i] = q[i];
    }



  //==========================================================================
  //  インタプリタを起動
  //==========================================================================


    for(index_t y=0; y < Screen.yMax; y++){

      for(index_t x=0; x < Screen.xMax; x++)
        print(" ");
    }

    Screen.x  = 0;
    Screen.y  = 0;

    asm_startupInterpreter(INTERPRETER_ADDRESS, GDT_USER_CODE_SELECTOR, GDT_USER_DATA_SELECTOR,
                           INTERPRETER_STACK_ADDRESS, INTEPRETER_RFLAGS);
}

